;---------------------------------------------------------------------------]
;-- WindowProcess 1.0 Library
;-- Author      : Prof. DrAcULA
;-- Date        : 6/2/2005 10:46PM
;-- Description : This library contains useful functions that let you
;--               play around with Windows Processes. All you need is
;--               a listbox on your Window that'll be used to display
;--               the processes. You can display and kill processes.
;--               Enjoy!!!
;---------------------------------------------------------------------------]

.386
.model flat, stdcall
option casemap:none

;---------------------------------------------------------------------------]
;--includes and libraries for WindowsProcess
        include         \masm32\include\windows.inc
        include         \masm32\include\user32.inc
        include         \masm32\include\kernel32.inc
        includelib      \masm32\lib\user32.lib
        includelib      \masm32\lib\kernel32.lib
;---------------------------------------------------------------------------]


;---------------------------------------------------------------------------]
;--local function prototypes
ListProcess     PROTO   :DWORD, :DWORD
KillProcess     PROTO   :DWORD, :DWORD
dwtoa           PROTO   :DWORD, :DWORD
DelString       PROTO   :DWORD, :DWORD, :DWORD
Sl              PROTO   :DWORD
;---------------------------------------------------------------------------]


.DATA
;---------------------------------------------------------------------------]
szBreak         DB      " ",0
szHeader        DB      " Process ID ---------------- Threads ------------- Process File ",0
szFilter        DB      " 0x%.5Xh                        %.3d                       %s",0
;---------------------------------------------------------------------------]


.DATA?
;---------------------------------------------------------------------------]
        WinProcess      PROCESSENTRY32  <>
        hSnapshot       DD              ?
        Buffer          DB              255 DUP(?)
        Process         DB              255 DUP(?)
;---------------------------------------------------------------------------]


.CODE
;---------------------------------------------------------------------------]
;--ListProcess fills a listbox, with the list of currently running
;--processes. Process-ID(in hex), Number of threads and process-filename
;--is shown in the listbox.
;--     Parameters: [1]. Handle of the listbox to be filled with process-info
;--                 [2]. Address of buffer that'll store number of active pr.
;---------------------------------------------------------------------------]
ListProcess     PROC    hList :DWORD, NumberProcess :DWORD
        LOCAL   Result  :DWORD

        mov     Result, 0
        invoke  SendMessage, [hList], LB_RESETCONTENT, 0, 0
        invoke  SendMessage, [hList], LB_ADDSTRING, 0, offset szHeader
        invoke  SendMessage, [hList], LB_ADDSTRING, 0, offset szBreak
        invoke  CreateToolhelp32Snapshot, TH32CS_SNAPPROCESS, 0
        mov     [hSnapshot], eax
        mov     [WinProcess.dwSize], sizeof PROCESSENTRY32
        invoke  Process32First, eax, offset WinProcess
        jmp     _GetRunningApps
  _Loop:
        inc     Result
        invoke  wsprintf, offset Buffer, offset szFilter, \
                [WinProcess.th32ProcessID], [WinProcess.cntThreads], \
                offset WinProcess.szExeFile
        invoke  SendMessage, [hList], LB_ADDSTRING, 0, offset Buffer
        invoke  Process32Next, [hSnapshot], offset WinProcess                
  _GetRunningApps:
        test    eax, eax
        jnz     _Loop
        invoke  CloseHandle, [hSnapshot]
        invoke  dwtoa, Result, NumberProcess
        xor     eax, eax
        ret
ListProcess     ENDP
;---------------------------------------------------------------------------]


;---------------------------------------------------------------------------]
;--KillProcess-function kills a process selected by user in the listbox.
;--After killing the specified process, it reloads the list of active
;--processes in the listbox.
;--     Parameters: [1]. Handle to the listbox
;--                 [2]. Address of buffer that'll store number of active pr.
;---------------------------------------------------------------------------]
KillProcess     PROC    hList :DWORD, NumberProcess :DWORD
        invoke  SendMessage, [hList], LB_GETCURSEL, 0, 0
        .if eax!=LB_ERR
                invoke  SendMessage, [hList], LB_GETTEXT, eax, offset Process
                invoke  DelString, offset Process, 0, 59
                invoke  CreateToolhelp32Snapshot, TH32CS_SNAPPROCESS, 0
                mov     [hSnapshot], eax
                mov     [WinProcess.dwSize], sizeof PROCESSENTRY32
                invoke  Process32First, eax, offset WinProcess
                jmp     _GetRunningApps
  _Loop:
                invoke  lstrcmp, offset Process, offset WinProcess.szExeFile
                .if eax==0
                        lea     edi, WinProcess
                        invoke  OpenProcess, PROCESS_TERMINATE, FALSE, [WinProcess.th32ProcessID]
                        push    eax
                        invoke  TerminateProcess, eax, NULL
                        pop     eax
                        invoke  Sleep, 50
                        invoke  CloseHandle, eax
                        invoke  CloseHandle, [hSnapshot]
                        invoke  ListProcess, hList, NumberProcess
                        xor     eax, eax
                        ret
                .else                        
                        invoke  Process32Next, [hSnapshot], offset WinProcess
                .endif                        
  _GetRunningApps:
                test    eax, eax
                jnz     _Loop
                invoke  CloseHandle, [hSnapshot]
                invoke  ListProcess, hList, NumberProcess
        .endif
        xor     eax, eax                
        ret
KillProcess     ENDP        
;---------------------------------------------------------------------------]


;---------------------------------------------------------------------------]
DelString       PROC    string:DWORD,from:DWORD,count:DWORD
        push    ecx
        push    esi
        push    edi
        mov     esi,string
        mov     edi,string
        push    string
        call    Sl
        add     esi,from
        add     esi,count
        add     edi,from
        dec     edi
        dec     esi
        mov     ecx,count
        sub     eax,from
        add     ecx,eax
        rep     movsb
        push    string
        call    Sl
        pop     edi
        pop     esi
        pop     ecx
        ret
DelString       ENDP

Sl      PROC    lpSource:DWORD
        push    edi
        push    ecx
        xor     eax,eax
        mov     edi,lpSource
        lea     ecx,[eax-1]
        repnz   scasb
        not     ecx
        lea     eax,[ecx-1]
        pop     ecx
        pop     edi
        ret
Sl      ENDP
;---------------------------------------------------------------------------]


;---------------------------------------------------------------------------]
dwtoa   PROC    dwValue :DWORD,lpBuffer :DWORD
        push    ebx
        push    esi
        push    edi
        mov     eax, dwValue
        mov     edi, [lpBuffer]
        or      eax, eax
        jnz     sign
    
  zero:
        mov     word ptr [edi], 30h
        jmp     dw2asc
 
  sign:
        jns     pos
        mov     byte ptr [edi], '-'
        neg     eax
        inc     edi
        
  pos:      
        mov     ecx, 429496730
        mov     esi, edi
        .while (eax > 0)
                mov     ebx, eax
                mul     ecx
                mov     eax, edx
                lea     edx, [edx*4+edx]
                add     edx, edx
                sub     ebx, edx
                add     bl, '0'
                mov     [edi], bl
                inc     edi
        .endw
        mov     byte ptr [edi], 0
        ; We now have all the digits, but in reverse order.
        .while (esi < edi)
                dec     edi
                mov     al, [esi]
                mov     ah, [edi]
                mov     [edi], al
                mov     [esi], ah
                inc     esi
        .endw
        
  dw2asc:
                pop     edi
                pop     esi
                pop     ebx
                ret
dwtoa   ENDP
;---------------------------------------------------------------------------]

END
;---------------------------------------------------------------------------]
;-- 2005, Prof. DrAcULA : "Knowledge is the sweetest blood!"
;---------------------------------------------------------------------------]